﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace PxToEm
{
    class Program
    {
        /// <summary>
        /// 路径分类
        /// </summary>
        public enum RouteType
        {
            // <summary>
            /// 仅文件名
            /// </summary>
            NameOnly = 0,
            /// <summary>
            /// 文件名（包括后缀）
            /// </summary>
            FileName = 1,
            /// <summary>
            /// 仅后缀名（包括.）
            /// </summary>
            FileSuffix = 2,
            /// <summary>
            /// 除文件名之外的路径
            /// </summary>
            PathExceptName = 3,
            /// <summary>
            /// 除后缀之外的路径
            /// </summary>
            PathExceptSuffix = 4
        }

        /// <summary>
        /// 程序入口Main
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            string oldFile, newFile;

            if (args.Length == 1)
            {
                oldFile = args[0];
                newFile = GetPartRoute(oldFile, RouteType.PathExceptName) + GetPartRoute(oldFile, RouteType.NameOnly) + "_New" + GetPartRoute(oldFile, RouteType.FileSuffix);
            }
            else if (args.Length == 2)
            {
                oldFile = args[0];
                newFile = args[1];
            }
            else
            {
                Console.WriteLine("Too many or few parameters,the correct format is:oldfile newfile"); return;
            }

            if (!File.Exists(oldFile))
            {
                Console.WriteLine("File does not exist");
            }
            else
            {
                List<string> newLine = new List<string>();

                foreach (var line in File.ReadLines(oldFile, ASCIIEncoding.UTF8))
                {
                    string pattern = @"(?<prex>(\:| ))(?<digt>[0-9]+)(?<cent>\s?)(?<unit>px)";

                    Match result = Regex.Match(line, pattern, RegexOptions.None);

                    if (!result.Success)
                    {
                        newLine.Add(line);
                    }
                    else
                    {
                        double digital = Math.Round(Convert.ToDouble(result.Result("${digt}")) * 0.0625, 2);
                        newLine.Add(Regex.Replace(line, pattern, "${prex}" + digital + "${cent}em", RegexOptions.None));
                    }
                }

                if (!File.Exists(newFile))
                {
                    FileStream fs = File.Create(newFile); fs.Close();
                }

                File.WriteAllLines(newFile, newLine.AsEnumerable(), ASCIIEncoding.UTF8); Console.WriteLine("Done");
            }
        }

        /// <summary>
        /// 正则获取文件路径
        /// </summary>
        /// <param name="strRoute">完整文件名</param>
        /// <param name="rType">要获取路径类型</param>
        /// <returns></returns>
        public static string GetPartRoute(string strRoute, RouteType rType)
        {
            string strResult = string.Empty;

            Regex regex = new Regex(@"^(?<fpath>([a-zA-Z]:\\)([\s\.\-\w]+\\)*)?(?<fname>[_\s\w]+)(?<namext>(\.[\w]+)*)(?<suffix>\.[\w]+)");

            Match result = regex.Match(strRoute);

            if (result.Success)
            {
                switch (rType)
                {
                    case RouteType.NameOnly:
                        strResult = result.Result("${fname}") + result.Result("${namext}"); break;
                    case RouteType.FileName:
                        strResult = result.Result("${fname}") + result.Result("${namext}") + result.Result("${suffix}"); break;
                    case RouteType.FileSuffix:
                        strResult = result.Result("${suffix}"); break;
                    case RouteType.PathExceptName:
                        strResult = result.Result("${fpath}"); break;
                    case RouteType.PathExceptSuffix:
                        strResult = result.Result("${fpath}") + result.Result("${fname}") + result.Result("${namext}"); break;
                    default:
                        strResult = strRoute; break;
                }
            }

            return strResult;
        }
    }
}
